<?php
/*
 * Plugin Name: WP Multi Comment Notifications
 * Plugin URI: https://git.viper.ipv64.net/M_Viper/wp-multi-comment-notifications
 * Description: Benachrichtigt bei neuen Kommentaren per E-Mail und optional über Telegram & Discord. Ideal für Teams, die schnell informiert werden wollen.
 * Version: 1.2
 * Author: M_Viper
 * Author URI: https://m-viper.de
 * Requires at least: 6.7.2
 * Tested up to: 6.7.2
 * License: GPL2
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: wp-multi-comment-notifications
 * Domain Path: /languages
 * Tags: kommentare, benachrichtigung, e-mail, telegram, discord, kommentarüberwachung, teamkommunikation
 *
 * Support:
 * - Microsoft Teams: https://teams.live.com/l/community/FEAzokphpZTJ2u6OgI
 * - Telegram: https://t.me/M_Viper04
 */

if (!defined('ABSPATH')) {
    exit;
}

define('WPMCN_PLUGIN_VERSION', '1.2');
define('WPMCN_PLUGIN_DIR', plugin_dir_path(__FILE__));

// Textdomain laden
function wpmcn_load_textdomain() {
    load_plugin_textdomain('wp-multi-comment-notifications', false, dirname(plugin_basename(__FILE__)) . '/languages');
}
add_action('plugins_loaded', 'wpmcn_load_textdomain');

// Dashboard Widget – Update Info für dieses Plugin
function wpmcn_update_dashboard_widget() {
    wp_add_dashboard_widget(
        'wpmcn_update_widget',
        __('Verfügbare Updates für WP Multi Comment Notifications', 'wp-multi-comment-notifications'),
        'wpmcn_update_dashboard_widget_content'
    );
}
add_action('wp_dashboard_setup', 'wpmcn_update_dashboard_widget');

// Benutzerdefiniertes Cron-Intervall hinzufügen
add_filter('cron_schedules', function ($schedules) {
    $schedules['hourly'] = array(
        'interval' => 3600, // 1 Stunde in Sekunden
        'display'  => __('Stündlich', 'wp-multi-comment-notifications'),
    );
    return $schedules;
});

// Cron-Job registrieren
function wpmcn_update_schedule_check() {
    if (!wp_next_scheduled('wpmcn_update_check_event')) {
        wp_schedule_event(time(), 'hourly', 'wpmcn_update_check_event');
    }
}
add_action('wp', 'wpmcn_update_schedule_check');

// Hilfsfunktion zur Wiederverwendung der Gitea-API-Abfrage
function wpmcn_fetch_latest_release($show_prereleases = false) {
    $api_url = 'https://git.viper.ipv64.net/api/v1/repos/M_Viper/wp-multi-comment-notifications/releases';
    $response = wp_remote_get($api_url, array('timeout' => 10));

    if (is_wp_error($response)) {
        error_log('WP Multi Comment Notifications Update Check Fehler: ' . $response->get_error_message());
        return false;
    }

    $body = wp_remote_retrieve_body($response);
    $data = json_decode($body, true);

    if (!is_array($data)) {
        error_log('WP Multi Comment Notifications Update Check: Ungültige API-Antwort');
        return false;
    }

    foreach ($data as $release) {
        if (!$show_prereleases && isset($release['prerelease']) && $release['prerelease']) {
            continue;
        }
        if (!empty($release['tag_name'])) {
            return $release;
        }
    }
    return null;
}

// Cron-Job für Update-Überprüfung
function wpmcn_update_check() {
    if (!function_exists('get_plugin_data')) {
        require_once ABSPATH . 'wp-admin/includes/plugin.php';
    }

    $plugin_data = get_plugin_data(__FILE__);
    $installed_version = $plugin_data['Version'];
    $show_prereleases = get_option('wpmcn_update_show_prereleases', false);

    $valid_release = wpmcn_fetch_latest_release($show_prereleases);

    if ($valid_release) {
        $latest_version = $valid_release['tag_name'];
        $release_notes = isset($valid_release['body']) ? $valid_release['body'] : '';
        $is_prerelease = isset($valid_release['prerelease']) && $valid_release['prerelease'];

        update_option('wpmcn_update_latest_version', $latest_version);
        update_option('wpmcn_update_release_notes', $release_notes);
        update_option('wpmcn_update_is_prerelease', $is_prerelease);
    }
}
add_action('wpmcn_update_check_event', 'wpmcn_update_check');

// Callback-Funktion für das Widget
function wpmcn_update_dashboard_widget_content() {
    if (!function_exists('get_plugin_data')) {
        require_once ABSPATH . 'wp-admin/includes/plugin.php';
    }

    $plugin_data = get_plugin_data(__FILE__);
    $installed_version = $plugin_data['Version'];
    $show_prereleases = get_option('wpmcn_update_show_prereleases', false);

    $valid_release = wpmcn_fetch_latest_release($show_prereleases);

    if ($valid_release === false) {
        printf(
            '<p style="color: red;">%s</p>',
            esc_html__('Fehler beim Abrufen der Versionsinformationen von Gitea.', 'wp-multi-comment-notifications')
        );
        return;
    }

    if ($valid_release) {
        $latest_version = $valid_release['tag_name'];
        $release_notes = isset($valid_release['body']) ? $valid_release['body'] : '';
        $is_prerelease = isset($valid_release['prerelease']) && $valid_release['prerelease'];

        if (version_compare($installed_version, $latest_version, '>=')) {
            printf(
                '<p style="color: green;">%s</p>',
                sprintf(
                    __('Ihre Version ist aktuell. Version %s ist die neueste Version.', 'wp-multi-comment-notifications'),
                    esc_html($installed_version)
                )
            );
        } else {
            printf(
                '<p style="color: red;">%s</p>',
                sprintf(
                    __('Es ist eine neue Version von WP Multi Comment Notifications verfügbar! <strong>Version %s</strong> ist jetzt verfügbar.', 'wp-multi-comment-notifications'),
                    esc_html($latest_version)
                )
            );
            printf(
                '<p>%s: <strong>%s</strong></p>',
                __('Aktuell installierte Version', 'wp-multi-comment-notifications'),
                esc_html($installed_version)
            );
            printf(
                '<p>%s: <strong>%s</strong></p>',
                __('Neue Version auf Gitea', 'wp-multi-comment-notifications'),
                esc_html($latest_version)
            );

            if ($is_prerelease && $show_prereleases) {
                printf(
                    '<p style="color: blue;">%s</p>',
                    __('Dieses Update ist ein PreRelease.', 'wp-multi-comment-notifications')
                );
            }

            if (!empty($release_notes)) {
                printf(
                    '<p><strong>%s:</strong></p><p>%s</p>',
                    __('Information zum Update', 'wp-multi-comment-notifications'),
                    nl2br(esc_html($release_notes))
                );
            }

            $button_text = $is_prerelease ? __('PreRelease herunterladen', 'wp-multi-comment-notifications') : __('Update herunterladen', 'wp-multi-comment-notifications');
            $download_url = isset($valid_release['assets'][0]['browser_download_url']) ? $valid_release['assets'][0]['browser_download_url'] : '#';
            printf(
                '<p><a href="%s" class="button button-primary">%s</a></p>',
                esc_url($download_url),
                esc_html($button_text)
            );
        }
    } else {
        printf(
            '<p>%s</p>',
            esc_html__('Keine Versionsinformationen gefunden.', 'wp-multi-comment-notifications')
        );
    }
}

// Füge die Checkbox zu den allgemeinen Einstellungen hinzu
function wpmcn_update_general_settings() {
    add_settings_section(
        'wpmcn_update_section',
        __('WP Multi Comment Notifications Update Einstellungen', 'wp-multi-comment-notifications'),
        null,
        'general'
    );

    add_settings_field(
        'wpmcn_update_show_prereleases',
        __('Pre-Releases anzeigen', 'wp-multi-comment-notifications'),
        'wpmcn_update_show_prereleases_callback',
        'general',
        'wpmcn_update_section'
    );

    register_setting('general', 'wpmcn_update_show_prereleases', array(
        'type' => 'boolean',
        'description' => __('Aktivieren, um Pre-Releases im Dashboard und in den Versionsinformationen anzuzeigen.', 'wp-multi-comment-notifications'),
        'default' => 0,
    ));
}
add_action('admin_init', 'wpmcn_update_general_settings');

// Callback-Funktion für das Anzeigen der Checkbox
function wpmcn_update_show_prereleases_callback() {
    $checked = get_option('wpmcn_update_show_prereleases', false);
    echo '<input type="checkbox" name="wpmcn_update_show_prereleases" value="1" ' . checked(1, $checked, false) . '/>';
    echo '<p style="color: red;"><small>' . __('Achtung: Pre-Releases sind Beta-Versionen und können Fehler enthalten. Verwenden Sie sie nur, wenn Sie Fehlerberichterstattung oder Tests durchführen möchten.', 'wp-multi-comment-notifications') . '</small></p>';
}

// Cleanup bei Deinstallation
function wpmcn_uninstall() {
    delete_option('wpmcn_update_latest_version');
    delete_option('wpmcn_update_release_notes');
    delete_option('wpmcn_update_is_prerelease');
    delete_option('wpmcn_update_show_prereleases');
    wp_clear_scheduled_hook('wpmcn_update_check_event');
}
register_uninstall_hook(__FILE__, 'wpmcn_uninstall');


// 1. Plugin-Optionen im Admin-Bereich unter dem Menü "Kommentare" hinzufügen
function wp_multi_comment_notifications_menu() {
    add_submenu_page(
        'edit-comments.php',
        'WP Multi Comment Notifications',
        'Kommentar Benachrichtigungen',
        'manage_options',
        'wp-multi-comment-notifications',
        'wp_multi_comment_notifications_settings_page'
    );
}
add_action('admin_menu', 'wp_multi_comment_notifications_menu');

function wp_multi_comment_notifications_settings_page() {
    ?>
    <div class="wrap">
        <h1>WP Multi Comment Notifications Einstellungen</h1>
        <form method="post" action="options.php">
            <?php
            settings_fields( 'wp_multi_comment_notifications_settings_group' );
            do_settings_sections( 'wp-multi-comment-notifications' );
            ?>
            <h3>Telegram Bot Token</h3>
            <label for="wp_telegram_token">Telegram Bot Token</label>
            <input type="text" name="wp_telegram_token" value="<?php echo esc_attr(get_option('wp_telegram_token')); ?>" class="regular-text" />

            <h3>Discord Webhook URL</h3>
            <label for="wp_discord_webhook_url">Webhook URL</label>
            <input type="text" name="wp_discord_webhook_url" value="<?php echo esc_attr(get_option('wp_discord_webhook_url')); ?>" class="regular-text" />

            <h3>Nachricht für Discord</h3>
            <textarea name="wp_discord_message" rows="5" class="large-text"><?php echo esc_textarea(get_option('wp_discord_message')); ?></textarea>
            <p>Verwende Platzhalter wie <code>{{COMMENT_AUTHOR}}</code>, <code>{{COMMENT_TEXT}}</code>, <code>{{USER_NAME}}</code> für dynamische Daten.</p>


            <h3>Nachricht für E-Mail</h3>
            <textarea name="wp_email_message" rows="5" class="large-text"><?php echo esc_textarea(get_option('wp_email_message')); ?></textarea>
            <p>Verwende Platzhalter wie <code>{{COMMENT_AUTHOR}}</code>, <code>{{COMMENT_TEXT}}</code>, <code>{{USER_NAME}}</code> für dynamische Daten.</p>

            <h3>Nachricht für Telegram</h3>
            <textarea name="wp_telegram_message" rows="5" class="large-text"><?php echo esc_textarea(get_option('wp_telegram_message')); ?></textarea>
            <p>Verwende Platzhalter wie <code>{{COMMENT_AUTHOR}}</code>, <code>{{COMMENT_TEXT}}</code>, <code>{{USER_NAME}}</code> für dynamische Daten.</p>

            <h3>Backend-Button URL</h3>
            <label for="wp_backend_button_url">URL für Backend Button</label>
            <input type="text" name="wp_backend_button_url" value="<?php echo esc_attr(get_option('wp_backend_button_url')); ?>" class="regular-text" />

            <h3>Logo/Banner für E-Mail</h3>
            <label for="wp_email_logo_url">Logo/Banner URL</label>
            <input type="text" name="wp_email_logo_url" value="<?php echo esc_attr(get_option('wp_email_logo_url')); ?>" class="regular-text" />
            <p>Füge hier die vollständige URL zum Logo/Banner ein, das in der E-Mail angezeigt werden soll.</p>

            <div class="user-boxes">
                <?php for ($i = 1; $i <= 6; $i++) : ?>
                    <div class="user-box">
                        <h3>User <?php echo $i; ?></h3>
                        <label for="wp_user_<?php echo $i; ?>_name">Name</label>
                        <input type="text" name="wp_user_<?php echo $i; ?>_name" value="<?php echo esc_attr(get_option("wp_user_{$i}_name")); ?>" />

                        <label for="wp_user_<?php echo $i; ?>_email">E-Mail Adresse</label>
                        <input type="email" name="wp_user_<?php echo $i; ?>_email" value="<?php echo esc_attr(get_option("wp_user_{$i}_email")); ?>" />

                        <label for="wp_user_<?php echo $i; ?>_telegram_chat_id">Telegram Chat ID</label>
                        <input type="text" name="wp_user_<?php echo $i; ?>_telegram_chat_id" value="<?php echo esc_attr(get_option("wp_user_{$i}_telegram_chat_id")); ?>" />

                        <!-- Button für individuelle Testnachricht -->
                        <input type="submit" name="send_test_message_user_<?php echo $i; ?>" class="button button-secondary" value="Testnachricht an <?php echo esc_attr(get_option("wp_user_{$i}_name")); ?>" />
                    </div>
                <?php endfor; ?>
            </div>
            <?php submit_button(); ?>
        </form>
    </div>

    <style>
        .user-boxes {
            display: flex;
            flex-wrap: wrap;
            gap: 20px;
        }

        .user-box {
            width: 30%;
            padding: 15px;
            border: 1px solid #ddd;
            border-radius: 5px;
            box-shadow: 0 2px 5px rgba(0, 0, 0, 0.1);
        }

        .user-box h3 {
            margin-top: 0;
        }

        .user-box label {
            display: block;
            margin-bottom: 5px;
        }

        .user-box input,
        .user-box textarea {
            width: 100%;
            padding: 8px;
            margin-bottom: 10px;
            border: 1px solid #ddd;
            border-radius: 5px;
        }

        .regular-text {
            width: 100%;
        }

        .large-text {
            width: 100%;
            height: 150px;
        }
    </style>
    <?php
}

function wp_multi_comment_notifications_register_settings() {
    for ($i = 1; $i <= 6; $i++) {
        register_setting('wp_multi_comment_notifications_settings_group', "wp_user_{$i}_name");
        register_setting('wp_multi_comment_notifications_settings_group', "wp_user_{$i}_email");
        register_setting('wp_multi_comment_notifications_settings_group', "wp_user_{$i}_telegram_chat_id");
    }
    register_setting('wp_multi_comment_notifications_settings_group', 'wp_telegram_token');
    register_setting('wp_multi_comment_notifications_settings_group', 'wp_email_message');
    register_setting('wp_multi_comment_notifications_settings_group', 'wp_telegram_message');
    register_setting('wp_multi_comment_notifications_settings_group', 'wp_backend_button_url');
    register_setting('wp_multi_comment_notifications_settings_group', 'wp_email_logo_url');
    register_setting('wp_multi_comment_notifications_settings_group', 'wp_discord_webhook_url');
    register_setting('wp_multi_comment_notifications_settings_group', 'wp_discord_message');

}
add_action('admin_init', 'wp_multi_comment_notifications_register_settings');

/** -------------------------
 * Kommentar-Benachrichtigung
 * ------------------------- */
function wp_multi_comment_notifications_on_comment($comment_ID) {
    $comment = get_comment($comment_ID);
    $author = $comment->comment_author;
    $text = $comment->comment_content;
    $site = get_bloginfo('name');
    $url = get_option('wp_backend_button_url');
    $logo = esc_url(get_option('wp_email_logo_url'));
    $discord_webhook = get_option('wp_discord_webhook_url');
    $discord_template = get_option('wp_discord_message');

    // Nachricht für Discord nur einmal senden
    if ($discord_webhook && $discord_template) {
        $discord_msg = strtr($discord_template, [
            '{{COMMENT_AUTHOR}}' => $author,
            '{{COMMENT_TEXT}}'   => $text,
        ]);
        wp_multi_comment_notifications_send_discord($discord_webhook, $discord_msg);
    }

    // E-Mail & Telegram Benachrichtigungen pro Benutzer
    for ($i = 1; $i <= 6; $i++) {
        $name = get_option("wp_user_{$i}_name");
        $email = get_option("wp_user_{$i}_email");
        $chat_id = get_option("wp_user_{$i}_telegram_chat_id");

        // Nachricht für E-Mail & Telegram
        $replacements = [
            '{{COMMENT_AUTHOR}}' => $author,
            '{{COMMENT_TEXT}}'   => $text,
            '{{USER_NAME}}'      => $name,
        ];

        $email_msg = strtr(get_option('wp_email_message'), $replacements);
        if ($url) $email_msg .= "\n\nWeitere Details hier: $url";

        if ($email) {
            $html = $logo ? "<img src='$logo' style='max-width:100%;margin-bottom:20px;'><br>" : '';
            $html .= nl2br(esc_html($email_msg));
            $headers = [
			'Content-Type: text/html; charset=UTF-8',
			'From' => get_option( 'admin_email' ),
		];
            wp_mail($email, 'Neuer Kommentar', $html, $headers);
        }

        $tg_msg = strtr(get_option('wp_telegram_message'), $replacements);

        if ($chat_id) {
            wp_multi_comment_notifications_send_telegram(
                get_option('wp_telegram_token'),
                $chat_id,
                $comment_ID,
                $tg_msg
            );
        }
    }
}
add_action('wp_insert_comment', 'wp_multi_comment_notifications_on_comment', 10, 1);


/** -------------------------
 * Testnachrichten senden
 * ------------------------- */
add_action('admin_init', function () {
    if ($_SERVER['REQUEST_METHOD'] === 'POST') {
        for ($i = 1; $i <= 6; $i++) {
            if (isset($_POST["send_test_message_user_$i"])) {
                wp_multi_comment_notifications_send_test_message($i);
            }
        }
    }
});

function wp_multi_comment_notifications_send_test_message($user_number) {
    $name = get_option("wp_user_{$user_number}_name");
    $email = get_option("wp_user_{$user_number}_email");
    $chat_id = get_option("wp_user_{$user_number}_telegram_chat_id");
    $discord_webhook = get_option('wp_discord_webhook_url');
    $token = get_option('wp_telegram_token');
    $button_url = get_option('wp_backend_button_url');

    $message = "Dies ist eine Testnachricht für $name.";

    if ($email) {
        wp_mail($email, 'Testnachricht', $message);
    }

    if ($chat_id) {
        wp_multi_comment_notifications_send_telegram($token, $chat_id, 0, $message);
    }

    if ($discord_webhook) {
        wp_multi_comment_notifications_send_discord($discord_webhook, $message);
    }
}


/** -------------------------
 * Telegram-Nachricht mit Button
 * ------------------------- */
function wp_multi_comment_notifications_send_telegram($token, $chat_id, $comment_ID, $message) {
    $url = "https://api.telegram.org/bot$token/sendMessage";
    $button_url = get_option('wp_backend_button_url');

    $data = [
        'chat_id' => $chat_id,
        'text' => $message,
        'parse_mode' => 'HTML'
    ];

    if ($button_url) {
        $data['reply_markup'] = json_encode([
            'inline_keyboard' => [[['text' => 'Zum Backend', 'url' => $button_url]]]
        ]);
    }

    wp_remote_post($url, [
        'body' => $data,
        'timeout' => 10,
        'headers' => ['Content-Type' => 'application/x-www-form-urlencoded'],
    ]);
}


/** -------------------------
 * Discord-Nachricht senden
 * ------------------------- */
function wp_multi_comment_notifications_send_discord($webhook_url, $message) {
    $payload = json_encode(['content' => $message]);

    wp_remote_post($webhook_url, [
        'headers' => ['Content-Type' => 'application/json'],
        'body' => $payload,
        'timeout' => 10,
    ]);
}
